#!/usr/bin/env python3

import os
import argparse
import logging
from pathlib import Path

from iccore import process
from iccore import logging_utils
from iccore import runtime

from iccicd.deploy import PyPiContext, PythonPackage

logger = logging.getLogger(__name__)


def deploy(repo_dir: Path, pypi_ctx: PyPiContext):

    package = PythonPackage(repo_dir)
    package.build()
    package.upload(pypi_ctx)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("action", 
                        help="Action to perform")
    parser.add_argument("--repo_dir",
                        type=Path,
                        default=Path(os.getcwd()),
                        help="Path to the repo to be deployed")
    parser.add_argument("--dry_run",
                        type=bool,
                        default=False,
                        help="Dry run script - don't make real changes")
    parser.add_argument("--pypi_token",
                        type=str,
                        default="",
                        help="Token for uploading packages to PyPI")
    parser.add_argument("--use_test_pypi",
                        type=bool,
                        default=False,
                        help="Use the testpypi repository rather than production")

    args = parser.parse_args()

    runtime.ctx.set_is_dry_run(args.dry_run)
    logging_utils.setup_default_logger()

    if args.action == "deploy":
        logger.info("Doing deployment")

        pypi_ctx = PyPiContext(args.pypi_token, args.use_test_pypi)

        deploy(args.repo_dir, pypi_ctx)



from pathlib import Path
import logging

from iccore import process

logger = logging.getLogger(__name__)


class PyPiContext():

    def __init__(self, token: str, use_test_repo: bool) -> None:
        self.token = token
        self.use_test_repo = use_test_repo


class PythonPackage():

    def __init__(self, repo_path: Path) -> None:
        self.repo_path = repo_path

    def build(self):
        logger.info("Building Python package")
        output_dir = self.repo_path / "dist"
        cmd = f"python3 -m build --outdir {output_dir} {self.repo_path}"
        process.run(cmd)
        logger.info("Finished building Python package")

    def upload(self, pypi_context: PyPiContext):
        logger.info("Uploading Python package")

        if not pypi_context.token:
            raise RuntimeError("Provided PyPi token cannot be empty")

        repo = ""
        if pypi_context.use_test_repo:
            repo = " -r testpypi "

        token = f"-p {pypi_context.token}"
        upload_dir = self.repo_path / "dist"
        cmd = f"twine upload {upload_dir}/*  --non-interactive {token} {repo}"
        process.run(cmd)
        logger.info("Finished uploading Python package")